/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 2009 Toporov Konstantin. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 3 only ("GPL")  (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.gnu.org/copyleft/gpl.html  See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * You should include file containing license in each project.
 * If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * If you wish your version of this file to be governed by
 * the GPL Version 3, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [GPL Version 3] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under the GPL Version 3 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 3 code and therefore, elected the GPL
 * Version 3 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 *
 * Contributor(s):
 *
 * Portions Copyrighted 2010 Toporov Konstantin.
 */

package org.diacalc.mobile;

/**
 *
 * @author Toporov Konstantin <www.diacalc.org>
 */
import com.sun.lwuit.*;
import com.sun.lwuit.events.ActionEvent;
import com.sun.lwuit.events.ActionListener;
import com.sun.lwuit.events.DataChangedListener;
import com.sun.lwuit.events.SelectionListener;
import com.sun.lwuit.layouts.BorderLayout;
import com.sun.lwuit.layouts.GridLayout;
import com.sun.lwuit.list.ListCellRenderer;
import com.sun.lwuit.list.ListModel;
import com.sun.lwuit.plaf.Border;
import java.util.Vector;
import javax.microedition.midlet.MIDlet;
import org.diacalc.mobile.components.CoefEditor;
import org.diacalc.mobile.components.ContextMenu;
import org.diacalc.mobile.maths.CoefUnit;
import org.diacalc.mobile.maths.Factors;
import org.diacalc.mobile.maths.Str;
import org.diacalc.mobile.maths.Sugar;

public class CoefsForm  extends FormBase implements ActionListener{
    private DCJmobile master;
    
    private static final int N_CF_COMMAND = 30;
    private static final int E_CF_COMMAND = 31;
    private static final int D_CF_COMMAND = 32;

    private static final int YES_COMMAND = 36;
    private static final int NO_COMMAND = 37;

    private static final Command newCoefCommand = new Command("Новые коэф.", N_CF_COMMAND);
    private static final Command editCoefCommand = new Command("Изм. коэф.", E_CF_COMMAND);
    private static final Command delCoefCommand = new Command("Удал. коэф.", D_CF_COMMAND);
    

    private static final Command yesCommand = new Command("Да",YES_COMMAND);
    private static final Command noCommand = new Command("Нет",NO_COMMAND);

    private List coefsL;
    
    private ContextMenu contextMenuProduct = new ContextMenu();
    
    protected void execute(MIDlet m,boolean refresh) {
        if (contextMenuProduct.getCommandsSize()==0){
            contextMenuProduct.addCommand(editCoefCommand,this);
            contextMenuProduct.addCommand(delCoefCommand,this);
        }
        if (refresh){
            form.getContentPane().setLayout(new BorderLayout());

            master = (DCJmobile)m;
            
            Container top = new Container(new GridLayout(1,4));
            
            top.addComponent(createLabel("Время"));
            top.addComponent(createLabel("k1"));
            top.addComponent(createLabel("k2"));
            top.addComponent(createLabel("ЦЕИ"));
                        
            coefsL = new List(new CoefsListModel(master.getCoefs())){
                public void longPointerPress(int x, int y){
                    super.longPointerPress(x, y);
                    //Тут показываем контекстное меню.
                    if (contextMenuProduct.getCommandsSize()==0 ||
                            coefsL.getModel().getSize()==0 ||
                            coefsL.getSelectedIndex()<0 || 
                            coefsL.getSelectedIndex()>(coefsL.getModel().getSize()-1)) return;
                    //Container 
                    contextMenuProduct.showContextMenu( 
                            ((CoefUnit)((CoefsListModel)coefsL.getModel()).getItemAt(coefsL.getSelectedIndex())).getTimeString()
                            );
                }
            };
            coefsL.setRenderer(new CoefsRenderer());
            
            form.getContentPane().addComponent(BorderLayout.NORTH, top);
            
            form.getContentPane().addComponent(BorderLayout.CENTER, coefsL);
            
         
            form.addMenuCommand(newCoefCommand,this);
            form.getContentPane().setScrollableY(true);
        }
    }
    public void actionPerformed(ActionEvent ev){
        int id = ev.getCommand().getId();
        switch(id){
            case E_CF_COMMAND: editCoef(); break;
            case D_CF_COMMAND: deleteCoef(); break;
            case N_CF_COMMAND: addCoefs(); break;
        }
        
    }
    private void editCoef(){
        int row = coefsL.getSelectedIndex();
        if (row<0) return;
        //Создали копию
        CoefUnit cu = new CoefUnit( (CoefUnit)coefsL.getModel().getItemAt(row) );
        
        Container c = new CoefEditor(cu,master.getUser());
        if (YES_COMMAND==Dialog.show("Измен. коэф-ты",
                c,
                new Command[]{yesCommand,noCommand} ).getId()){
                cu = ((CoefEditor)c).getCoefUnit();
                
                ((CoefsListModel)coefsL.getModel()).changeItem(row, cu);
                master.setNeedSaveCoefs();
                if (master.getUser().isTimeSense()){
                    master.calcCoefsbyTime();
                    master.shouldRefreshMenu();
                }
        }
    }
    private void deleteCoef(){
        int row = coefsL.getSelectedIndex();
        if (row<0) return;
        coefsL.getModel().removeItem(row);
        
        master.setNeedSaveCoefs();
        if (master.getUser().isTimeSense()){
            master.calcCoefsbyTime();
            master.shouldRefreshMenu();
        }
    }
    private void addCoefs(){
        CoefUnit cu = new CoefUnit();
        Container c = new CoefEditor(cu,master.getUser());
        if (YES_COMMAND==Dialog.show("Новые коэф-ты",
                c,
                new Command[]{yesCommand,noCommand} ).getId()){
                cu = ((CoefEditor)c).getCoefUnit();
                coefsL.getModel().addItem(cu);
                
                master.setNeedSaveCoefs();
                if (master.getUser().isTimeSense()){
                    master.calcCoefsbyTime();
                    master.shouldRefreshMenu();
                }
        }
    }
    public String getIconName(){
        return "coefs";
    }
    public String getName(){
        return "Коэфф.";
    }
    protected String getHelp() {
        return "Помощь";
    }
    
    private Label createLabel(String txt){
        Label res = new Label(txt);
        res.getStyle().setAlignment(Label.CENTER);
        return res;
    }
    ////////////////////////////////
    class CoefsRenderer extends Container implements ListCellRenderer{
        private Label lblFocus = new Label();
        
        private Label labelTime = createLabel("");
        private Label labelk1 = createLabel("");
        private Label labelk2 = createLabel("");
        private Label labelk3 = createLabel("");
        private Sugar s = new Sugar();
        public CoefsRenderer(){
            super(new GridLayout(1,4));
            addComponent(labelTime);
            addComponent(labelk1);
            addComponent(labelk2);
            addComponent(labelk3);
            
            getStyle().setBorder(Border.createCompoundBorder(null,Border.createLineBorder(1), null, null));
        }
        public Component getListCellRendererComponent(List list, Object o, int i, boolean bln) {
            if (o==null) return lblFocus;
            CoefUnit cu = (CoefUnit)o;
            labelTime.setText(cu.getTimeString());
            labelk1.setText(
                    Str.addZero(Str.round(cu.getFactors().getK1(Factors.DIRECT),2)) );
            labelk2.setText(
                    Str.addZero(Str.round(cu.getFactors().getK2(),2))     );
            s.setValue(cu.getFactors().getK3());
            if (master.getUser().isMmol()){
                labelk3.setText(
                        Str.addZero(Str.round(
                    s.getSugar(master.getUser().isMmol(),
                        master.getUser().isPlasma()),2)));
            }else{
                labelk3.setText(""+Str.round(
                    s.getSugar(master.getUser().isMmol(),
                        master.getUser().isPlasma()),1));
            }
            
            return this;
        }
        public Component getListFocusComponent(List list) {
            return lblFocus;
        }
        /**
        * Overriden to do nothing and remove a performance issue where renderer changes
        * perform needless repaint calls
        */
        public void repaint() {
        }
    }
    
    class CoefsListModel implements ListModel{
        private Vector coefs;
        private int selIndx = -1;
        private Vector dataChangeListeners = new Vector();
        private Vector selectionChangeListeners = new Vector();
        
        public CoefsListModel(Vector coefs){
            this.coefs = coefs;
        }

        public Object getItemAt(int i) {
            if(i < getSize() && i >= 0){
                return coefs.elementAt(i);
            }
            return null;
        }

        public int getSize() {
            return coefs.size();
        }

        public int getSelectedIndex() {
            return selIndx;
        }

        public void setSelectedIndex(int i) {
            if (i>(coefs.size()-1)) return;
            int old_sel = selIndx;
            selIndx = i;
            fireSelectionChanged(old_sel, selIndx);
        }

        
        public void changeItem(int indx, Object o){//Тут индекс старая позиция, а о - новый экземпляр объекта
            removeItem(indx);
            addItem(o);
        }
        private int addCoefUnit(CoefUnit cu){
            int i=0;
            for(;i<coefs.size();i++){
                if ( ((CoefUnit)coefs.elementAt(i)).getTime()>cu.getTime() ){
                    break;
                }
            }
            coefs.insertElementAt(cu, i);
            //fireRowChanged(i);
            return i;
        }
        
        public void addItem(Object o) {
            //Добавляем и сразу соритруем, возвращаем индекс, куда добавили
            fireDataChanged(DataChangedListener.ADDED, addCoefUnit((CoefUnit)o));
        }

        public void removeItem(int i) {
            coefs.removeElementAt(i);
            fireDataChanged(DataChangedListener.REMOVED, i);
        }
        private void fireSelectionChanged(int o,int n){
            for (int i=0;i<selectionChangeListeners.size();i++){
                ((SelectionListener)selectionChangeListeners.elementAt(i)).selectionChanged(o, n);
            }
        }
        private void fireDataChanged(int type,int indx){
            for(int i=0;i<dataChangeListeners.size();i++){
                ((DataChangedListener)dataChangeListeners.elementAt(i)).dataChanged(type, indx);
            }
        }
        public void addDataChangedListener(DataChangedListener dl) {
            dataChangeListeners.addElement(dl);
        }

        public void removeDataChangedListener(DataChangedListener dl) {
            dataChangeListeners.removeElement(dl);
        }

        public void addSelectionListener(SelectionListener sl) {
            selectionChangeListeners.addElement(sl);
        }

        public void removeSelectionListener(SelectionListener sl) {
            selectionChangeListeners.removeElement(sl);
        }
    }
    ////////////////////////////////
}
