/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 2009 Toporov Konstantin. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 3 only ("GPL")  (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.gnu.org/copyleft/gpl.html  See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * You should include file containing license in each project.
 * If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * If you wish your version of this file to be governed by
 * the GPL Version 3, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [GPL Version 3] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under the GPL Version 3 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 3 code and therefore, elected the GPL
 * Version 3 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 *
 * Contributor(s):
 *
 * Portions Copyrighted 2010 Toporov Konstantin.
 */

package org.diacalc.mobile;


/**
 * Base class for a demo contains common code for all demo pages
 *
 * @author Shai Almog
 * @author Toporov Konstantin <www.diacalc.org>
 */

import com.sun.lwuit.events.DataChangedListener;
import com.sun.lwuit.events.SelectionListener;
import com.sun.lwuit.list.ListModel;
import java.util.Vector;
import org.diacalc.mobile.products.ProductInBase;


public class FilterProxyListModel implements ListModel, DataChangedListener {
static final String RUSUP = "АБВГДЕЁЖЗИЙКЛМНОПРСТУФХЦЧШЩЪЬЫЭЮЯ";
static final String RUSLO = "абвгдеёжзийклмнопрстуфхцчшщъьыэюя";

   private ListModel underlying;
   private Vector filter;
   private Vector listeners = new Vector();
   public FilterProxyListModel(ListModel underlying) {
       this.underlying = underlying;
       underlying.addDataChangedListener(this);
   }
   String toUpperRus(String s) {
    char[] c = new char[s.length()];
    for(int i=0; i < c.length;i++) {
     int j = RUSLO.indexOf( s.charAt(i));
     if ( j< 0)
       c[i] = s.charAt(i);
     else
       c[i] = RUSUP.charAt(j);
    }
    return new String(c);
   }
   private int getFilterOffset(int index) {
       if(filter == null) {
           return index;
       }
       if(filter.size() > index && index>=0) {
           return ((Integer)filter.elementAt(index)).intValue();
       }
       return -1;
   }

   private int getUnderlyingOffset(int index) {
       if(filter == null) {
           return index;
       }
       return filter.indexOf(new Integer(index));
   }

   public void filter(String str,int owner) {
       filter = new Vector();
       str = toUpperRus(str.toUpperCase());
       for(int iter = 0 ; iter < underlying.getSize() ; iter++) {
           ProductInBase element = (ProductInBase)underlying.getItemAt(iter);
           if( (element.getOwner()==owner || owner==-1) &&
                   (toUpperRus(element.getName().toUpperCase()).indexOf(str) > -1)) {
               filter.addElement(new Integer(iter));
           }
       }
       dataChanged(DataChangedListener.CHANGED, -1);
   }

   public Object getItemAt(int index) {
       if (index<0) return null;
       return underlying.getItemAt(getFilterOffset(index));
   }

   public int getSize() {
       if(filter == null) {
           return underlying.getSize();
       }
       return filter.size();
   }

   public int getSelectedIndex() {
       return Math.max(0, getUnderlyingOffset(underlying.getSelectedIndex()));
   }

   public void setSelectedIndex(int index) {
       underlying.setSelectedIndex(getFilterOffset(index));
   }

   public void addDataChangedListener(DataChangedListener l) {
       listeners.addElement(l);
   }

   public void removeDataChangedListener(DataChangedListener l) {
       listeners.removeElement(l);
   }

   public void addSelectionListener(SelectionListener l) {
       underlying.addSelectionListener(l);
   }

   public void removeSelectionListener(SelectionListener l) {
       underlying.removeSelectionListener(l);
   }

   public void addItem(Object item) {
       underlying.addItem(item);
   }

   public void removeItem(int index) {
       underlying.removeItem(index);
   }

   public void dataChanged(int type, int index) {
       if(index > -1) {
           index = getUnderlyingOffset(index);
           if(index < 0) {
               return;
           }
       }
       for(int iter = 0 ; iter < listeners.size() ; iter++) {
            ((DataChangedListener)listeners.elementAt(iter)).dataChanged(type, index);
       }
   }
}



