/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 2009 Toporov Konstantin. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 3 only ("GPL")  (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.gnu.org/copyleft/gpl.html  See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * You should include file containing license in each project.
 * If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * If you wish your version of this file to be governed by
 * the GPL Version 3, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [GPL Version 3] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under the GPL Version 3 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 3 code and therefore, elected the GPL
 * Version 3 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 *
 * Contributor(s):
 *
 * Portions Copyrighted 2010 Toporov Konstantin.
 */

package org.diacalc.mobile.components;

/**
 *
 * @author Toporov Konstantin <www.diacalc.org>
 */

import com.sun.lwuit.Component;
import com.sun.lwuit.TextField;
import com.sun.lwuit.events.ActionEvent;
import com.sun.lwuit.events.ActionListener;
import com.sun.lwuit.events.DataChangedListener;
import com.sun.lwuit.events.FocusListener;
import java.util.Hashtable;

public class MaskTextField extends TextField implements DataChangedListener{
    private String mask;//# - any digit all the rest is decor
    private char placeholder = '0';
    private String acceptedchars = "0123456789";
    private MaskValidator validator = null;
    
      
    public MaskTextField(String mask){
        super();
        this.mask = mask;
        initInputMode();
        this.setInputMode("123");
        this.setInputModeOrder(new String[] {"123"});
        setText("");
        
        addDataChangeListener(this);

        this.addFocusListener(new FocusListener(){
            public void focusGained(Component c){
                MaskTextField.this.setCursorPosition(0);
                int i=0;
                for(;i<MaskTextField.this.mask.length();i++){
                    if ( MaskTextField.this.mask.charAt(i)=='#' ){
                        //цифра, с этого места можно редактировать
                        break;
                    }
                }
                if (i==MaskTextField.this.mask.length()){
                    i=0;
                }
                MaskTextField.this.setCursorPosition(i);
            }
            public void focusLost(Component c){}
        });

        addActionListener(new ActionListener(){
            public void actionPerformed(ActionEvent ev){
                if (validator!=null){
                    validator.parse(MaskTextField.this);
                }
            }
        });
    }
    public void dataChanged(int type,int index){
        //вот тут проверяем ввод и корректируем его
        if (type==DataChangedListener.ADDED){
            char newch = getText().charAt(index);
            //проверяем подходит ли
            boolean found = false;
            for(int i=0;i<acceptedchars.length();i++){
                if (newch==acceptedchars.charAt(i)){
                    found = true;
                    break;
                }
            }
            if (!found){//не найден, удаляем
                setText(getText().substring(0, index) +
                        getText().substring(index+1));
            }
            else{ //Можно добавить, смотрим куда
                setText(check(index,getText()));
            }
        }else if (type==DataChangedListener.REMOVED){
            if (MaskTextField.this.mask.charAt(index)=='#')
                setText(getText().substring(0, index) + placeholder + getText().substring(index));
            else{
                setText(getText().substring(0, index) + MaskTextField.this.mask.charAt(index) + getText().substring(index));
                //setCursorPosition(getCursorPosition()>0?getCursorPosition()-1:0  );
            }
        }
    }

    /*private String validText(){
        System.out.println("validation text");
        //1-убираем все левые символы
        //flag = true;
        String st = "";
        String stin = getText();
        for(int i=0;i<stin.length();i++){
            for(int j=0;j<acceptedchars.length();j++){
                if (stin.charAt(i)==acceptedchars.charAt(j)){
                    st += stin.charAt(i);
                    break;
                }
            }
        }
        if (MaskTextField.this.mask==null){
            return "";
        }
        //А теперь заполняем по маске
        int j=0;
        String res = "";
        for(int i=0;i<MaskTextField.this.mask.length();i++){
            //Проверяем что за символ в маске
            if (MaskTextField.this.mask.charAt(i)=='#'){//Берем из вставляемой строки
                if (j<st.length()){//если есть что вставлять
                    res += st.charAt(j++);
                }
                else{//иначе заменяем заменителем
                    res += placeholder;
                }
            }
            else{//Берем символ из маски
                res += MaskTextField.this.mask.charAt(i);
            }
        }
        System.out.println("res="+res);

        return res;
    }*/

    public void setValidator(MaskValidator v){
        validator = v;
        //this.fireClicked();//Попробуем это исколькозовать
    }
    public String getMask(){
        return mask;
    }
    public void setValue(Object ob){
        if (validator!=null){
            setText(validator.getStringValue(this,ob));
        }
        else setText(ob.toString());
    }
    public Object getValue(){
        if (validator!=null){
            return validator.parse(this);
        }
        else return getText();
    }
    protected String check(int index, String toCheck){
        if (index<MaskTextField.this.mask.length()){
            if ( MaskTextField.this.mask.charAt(index)=='#' ){//добавляем
                if (index<(getText().length()-1) ) {
                    //Нужно заменить плейсхолдер новым символом
                    String st =  toCheck.substring(0, index+1) + toCheck.substring(index+2);
                    if (index<(MaskTextField.this.mask.length()-1) &&
                            MaskTextField.this.mask.charAt(index+1)!='#'){
                        setCursorPosition(getCursorPosition()+1);
                    }else setCursorPosition(index+1);
                    return st;
               }
               else{//Конец строки, дропаем введенный символ и возвращаем
                    return toCheck.substring(0, MaskTextField.this.mask.length() );
               }
            }else{//передвигаем на один символ вправо
                //
                char ch = toCheck.charAt(index);
                String st = toCheck.substring(0, index)+toCheck.substring(index+1);
                st = st.substring(0, index+1)+ch+toCheck.substring(index+2);
                return check(index+1,st);
            }
        }
        else{//строка кончилась, обрезаем
            return toCheck.substring(0, MaskTextField.this.mask.length() );
        }
    }
    protected boolean isAccepted(char c){
        for(int i=0;i<acceptedchars.length();i++){
            if (c==acceptedchars.charAt(i)) return true;
        }
        return false;
    }
    
    public void setPlaceholder(char v){
        placeholder = v;
        setText(getText());
    }
    public void setAcceptedchars(String chars){
        acceptedchars = chars;
    }
    private void initInputMode(){
        Hashtable hs = new Hashtable();
        hs.put(new Integer('1'), "1");//9
        hs.put(new Integer('2'), "2");//2
        hs.put(new Integer('3'), "3");//3
        hs.put(new Integer('4'), "4");//4
        hs.put(new Integer('5'), "5");//5
        hs.put(new Integer('6'), "6");//6
        hs.put(new Integer('7'), "7");//7
        hs.put(new Integer('8'), "8");//8
        hs.put(new Integer('9'), "9");//9
        hs.put(new Integer('0'), "0");//9

        addInputMode("123", hs, false);
    }
}
